const { sign } = require('../../services/jwt');
const { success } = require('../../services/response');
const { loggerError } = require('../../services/logger');
const { addNewUserToDB, getAllUsersDataFromDB } = require('../../utils/data-handlers');


const login = ({ user, ...other }, res, next) =>
  sign(user)
    .then((token) => ({ token, user }))
    .then(success(res, 201))
    .catch(loggerError(next, other))

const register = async ({ body, ...other }, res, next) => {
  const { email, password } = body;
  try {
    const allUsers = await getAllUsersDataFromDB();
    const isUserAlreadyExist = allUsers.some(el => el.email === email);
    if (isUserAlreadyExist) return res.status(403).send({ message: 'User already exists'})

    const newUser = await addNewUserToDB({ email, password });
    return success(res, 201)(newUser.view)
  } catch (error) {
    return loggerError(next, other)(error)
  }
}

module.exports = {
  login,
  register
}