const { uuid } = require('uuidv4');
const crypto = require('crypto');
const bcrypt = require('bcryptjs');

class User {
  constructor({ email, password, createAt, updateAt, id, picture }, createMode = true) {
    const timeStamp = new Date().toISOString();
    const hash = crypto.createHash('md5').update(email).digest('hex');
    const salt = bcrypt.genSaltSync(10);
    
    this.email = email;
    this.password = createMode ? bcrypt.hashSync(password, salt) : password;
    this.createAt = createMode ? timeStamp : createAt;
    this.updateAt = createMode ? timeStamp : updateAt;
    this.id = createMode ? uuid() : id;
    this.picture = createMode ? `https://gravatar.com/avatar/${hash}?d=identicon` : picture;
  }

  get view() {
    return {
      email: this.email,
      createAt: this.createAt,
      updateAt: this.updateAt,
      id: this.id,
      picture: this.picture,
    }
  }

  authenticate(password) {
    return bcrypt.compareSync(password, this.password);
  } 
}

module.exports = {
  User
}