const Joi = require('joi');

const newUserSchema = Joi.object().keys({ 
  password: Joi.string().min(6).required(),
  email: Joi.string().email().required(),
});

const joiMiddleware = (schema) => { 
  return (req, res, next) => { 
  const { error } = schema.validate(req.body); 

  if (!error) { 
    return next(); 
  } else { 
    const { details } = error; 
    const message = details.map(i => i.message).join(',');

   return res.status(422).json({ error: message }) } 
  } 
}

module.exports = {
  newUserSchema,
  joiMiddleware
};
