
const { createLogger, transports, format } = require('winston');

const { combine, printf, timestamp } = format

const options = {
  error: {
    level: 'error',
    filename: './logs/errors.log',
    handleExceptions: true,
    json: true,
    maxsize: 5242880,
    maxFiles: 5
  }
}

const logger = createLogger({
  format: combine(
    timestamp({ format: 'YYYY-MM-DD HH:mm:ss:ms' }),
    printf(info => `${info.timestamp} ${info.level}: ${info.message} ${info.location}`)
  ),
  transports: [
    new transports.File(options.error),
    new transports.Console()
  ]
})

const loggerError = (next, data) => (err) => {
  if (data) {
    const { headers, method, url } = data

    logger.error(`URL: ${headers.referer}${url}, Method: ${method}, ErrorMessage: ${err.message}`)
  } else {
    logger.error(err)
  }

  return next(err)
}

module.exports = {
  loggerError
}